<?php
/**
 * RW Elephant Invoice Class
 *
 * @author R.W. Elephant <support@rwelephant.com>
 *
 * @since 1.0.0
 */

namespace RWEG;

if ( ! defined( 'ABSPATH' ) ) {

	exit;

}

class Invoice {

	private $api_base;

	private $invoice_id;

	public static $invoice_data;

	public function __construct() {

		if ( ! is_rwe_invoice() ) {

			return;

		}

		// Setup invoice data
		$this->api_base   = sprintf( 'https://%s.rwelephant.com/perl/payment_api/', Options::$options['tenant-id'] );
		$this->invoice_id = get_rwe_invoice_id();

		$this->setup_invoice();

		self::$invoice_data = $this->request_invoice_data();

		if ( ! is_array( self::$invoice_data ) ) {

			return;

		}

		// Invoice AJAX handler
		add_action( 'wp_ajax_accept_terms', [ $this, 'process_accept_terms' ] );
		add_action( 'wp_ajax_nopriv_accept_terms', [ $this, 'process_accept_terms' ] );

		// Invoice AJAX handler
		add_action( 'wp_ajax_submit_payment', [ $this, 'process_submit_payment' ] );
		add_action( 'wp_ajax_nopriv_submit_payment', [ $this, 'process_submit_payment' ] );

		if ( ! $this->invoice_id || ( isset( self::$invoice_data['response_status'] ) && 'Error' === self::$invoice_data['response_status'] ) ) {

			return add_filter(
				'the_content',
				function( $content, $post_id = 0 ) {

					// Hide the terms.
				?><style>.invoice-terms { display: none; }</style><?php // @codingStandardsIgnoreLine

				return esc_html( self::$invoice_data['response_message'] );

				},
				10,
				2
			);

			return;

		}

		// Bail on non-https
		if ( ! is_ssl() ) {

			return add_filter(
				'the_content',
				function( $content, $post_id = 0 ) {

					return esc_html_e( 'Your site does not appear to be utilizing HTTPS. For security purposes, paying invoices will not be enabled until you have installed an SSL certificate and are serving content via HTTPS.', 'rw-elephant-rental-inventory' );

				},
				10,
				2
			);

		}

		// Bail on error
		if ( ! self::$invoice_data || is_wp_error( self::$invoice_data ) ) {

			return add_filter(
				'the_content',
				function( $content, $post_id = 0 ) {

					return self::$invoice_data->get_error_message();

				},
				10,
				2
			);

		}

		if ( ! $this->invoice_id || empty( $this->invoice_id ) ) {

			return add_filter(
				'the_content',
				function( $content, $post_id = 0 ) {

					return esc_html__( 'Missing or invalid invoice ID.', 'rw-elephant-rental-inventory' );

				},
				10,
				2
			);

		}

	}

	/**
	 * Request the invoice data.
	 *
	 * @since 2.0.0
	 *
	 * @return Error | jsonp data
	 */
	public function request_invoice_data() {

		$url = add_query_arg(
			[
				'action'            => 'invoice_detail',
				'invoice_public_id' => get_rwe_invoice_id(),
				'callback'          => 'data',
			],
			$this->api_base
		);

		$request = wp_remote_get(
			$url,
			[
				'timeout' => '30',
			]
		);

		$response_body = wp_remote_retrieve_body( $request );
		$response_code = wp_remote_retrieve_response_code( $request );

		if ( 200 !== $response_code ) {

			return new \WP_Error( 'error', __( 'An error occurred retreiving the invoice data, please refresh the page and try again.', 'rw-elephant-rental-inventory' ) );

		}

		// Store the invoice details
		$invoice_details = rwe_jsonp_decode( $response_body );

		$invoice_details['logo'] = '';

		// Append the URL to the company logo
		$invoice_details['logo'] = add_query_arg(
			[
				'action' => 'small_logo',
				'sid'    => isset( $invoice_details['sid'] ) ? $invoice_details['sid'] : '',
			],
			$this->api_base
		);

		return $invoice_details;

	}

	/**
	 * Process the accept terms page.
	 * Note: AJAX Handler
	 *
	 * @since 2.0.0
	 *
	 * @return mixed Markup for the payment page, else error.
	 */
	public function process_accept_terms() {

		check_ajax_referer( 'rw_elephant_accept_terms', '_wpnonce' );

		$process_signature = $this->process_signature( $_POST );

		if ( is_wp_error( $process_signature ) ) {

			wp_send_json_error( $process_signature->get_error_message() );

		}

		ob_start();

		Plugin::load_rw_template( 'invoice-pay' );

		$template = ob_get_contents();

		ob_get_clean();

		wp_send_json_success( $template );

	}

	/**
	 * Process the esignature data and send it to RWE servers
	 *
	 * @since 2.0.0
	 */
	private function process_signature( $post_data ) {

		$request = wp_remote_post(
			$this->api_base,
			[
				'timeout' => '30',
				'body'    => [
					'action'            => 'process_signature',
					'invoice_public_id' => filter_input( INPUT_POST, 'invoiceID', FILTER_SANITIZE_STRING ),
					'signed_by'         => filter_input( INPUT_POST, 'termName', FILTER_SANITIZE_STRING ),
					'signed_email'      => filter_input( INPUT_POST, 'termEmail', FILTER_VALIDATE_EMAIL ),
					'signature_image'   => filter_input( INPUT_POST, 'signaturePNG', FILTER_SANITIZE_STRING ),
				],
			]
		);

		$response_body = rwe_jsonp_decode( wp_remote_retrieve_body( $request ) );
		$response_code = wp_remote_retrieve_response_code( $request );

		// Error processing request on RW Elephant servers.
		if ( 200 !== $response_code ) {

			return new \WP_Error( 'error', __( 'Error processing your signature.', 'rw-elephant-rental-inventory' ) );

		}

		// Error in the request.
		if ( 'Error' === $response_body['response_status'] ) {

			return new \WP_Error( 'error', $response_body['response_message'] );

		}

		return true;

	}

	/**
	 * After a successful payment, submit the data to RW Elephants servers.
	 * Note: AJAX Handler
	 *
	 * @since 2.0.0
	 *
	 * @return boolean True when data is passed to RW elephant, else false.
	 */
	public function process_submit_payment() {

		check_ajax_referer( 'rw_elephant_submit_payment', '_wpnonce' );

		$stripe_token   = filter_input( INPUT_POST, 'stripeKey', FILTER_SANITIZE_STRING );
		$payment_amount = filter_input( INPUT_POST, 'paymentAmount', FILTER_SANITIZE_STRING );
		$charge_card    = filter_input( INPUT_POST, 'cardForBalance', FILTER_SANITIZE_NUMBER_INT );
		$stripe_email   = filter_input( INPUT_POST, 'stripeEmail', FILTER_SANITIZE_EMAIL );

		$url = add_query_arg(
			[
				'action'               => 'process_payment',
				'sid'                  => self::$invoice_data['sid'],
				'invoice_public_id'    => get_rwe_invoice_id(),
				'stripe_token'         => $stripe_token,
				'payment_amount'       => $payment_amount,
				'use_card_for_balance' => $charge_card,
				'stripeEmail'          => $stripe_email,
				'callback'             => 'data',
			],
			$this->api_base
		);

		$request = wp_remote_get(
			$url,
			[
				'timeout' => '30',
			]
		);

		$response_body = rwe_jsonp_decode( wp_remote_retrieve_body( $request ) );
		$response_code = wp_remote_retrieve_response_code( $request );

		// Error processing request on RW Elephant servers.
		if ( 200 !== $response_code ) {

			wp_send_json_error( __( 'Error processing your request. Please refresh the page, and try again.', 'rw-elephant-rental-inventory' ) );

		}

		// Error in the request.
		if ( 'Error' === $response_body['response_status'] ) {

			wp_send_json_error( $response_body['response_message'] );

		}

		ob_start();

		?>

		<div class="rwe-order-payment__payment-notice">
			<div class="rwe-payment-notice">
				<h2 class="rwe-payment-notice__title"><?php esc_html_e( 'Your payment has been successfully processed.', 'rw-elephant-rental-inventory' ); ?></h2>
			</div>
		</div>

		<div class="rwe-order-payment__payment-data">
			<table class="rwe-payment-data">
				<tr class="rwe-payment-data__row">
					<td class="rwe-payment-data__label" align="left"><?php esc_html_e( 'Invoice:', 'rw-elephant-rental-inventory' ); ?></td>
					<td class="rwe-payment-data__value" align="left"><?php echo esc_html( $response_body['event_date_dotted'] . ' ' . $response_body['client'] . ' ' . $response_body['display_invoice_id'] ); ?></td>
				</tr>
				<tr class="rwe-payment-data__row">
					<td class="rwe-payment-data__label" align="left"><?php esc_html_e( 'Amount Paid:', 'rw-elephant-rental-inventory' ); ?></td>
					<td class="rwe-payment-data__value" align="left"><?php echo esc_html( $response_body['this_payment_amount'] ); ?></td>
				</tr>
				<tr class="rwe-payment-data__row">
					<td class="rwe-payment-data__label" align="left"><?php esc_html_e( 'Payment Date:', 'rw-elephant-rental-inventory' ); ?></td>
					<td class="rwe-payment-data__value" align="left"><?php echo esc_html( $response_body['this_payment_date_dotted'] ); ?></td>
				</tr>
				<tr class="rwe-payment-data__row">
					<td class="rwe-payment-data__label" align="left"><?php esc_html_e( 'Method:', 'rw-elephant-rental-inventory' ); ?></td>
					<td class="rwe-payment-data__value" align="left"><?php esc_html_e( 'Credit/Debit Card', 'rw-elephant-rental-inventory' ); ?></td>
				</tr>
				<tr class="rwe-payment-data__row">
					<td class="rwe-payment-data__value" colspan="2" align="left"><?php esc_html_e( 'Please print this page for your records and check your inbox for an email from &quot;RW Elephant&quot; confirming your payment.', 'rw-elephant-rental-inventory' ); ?></td>
				</tr>
			</table>
		</div>

		<?php

		$success_markup = ob_get_contents();

		ob_get_clean();

		// Success.
		wp_send_json_success( $success_markup );

	}

	/**
	 * Get the invoice data.
	 * from /perl/payment_api?action=invoice_terms
	 *
	 * @return array Invoice details array.
	 */
	public static function get_invoice_data() {

		$url = add_query_arg(
			[
				'action'            => 'invoice_terms',
				'invoice_public_id' => get_rwe_invoice_id(),
				'callback'          => 'data',
			],
			sprintf( 'https://%s.rwelephant.com/perl/payment_api/', Options::$options['tenant-id'] )
		);

		$request = wp_remote_get(
			$url,
			[
				'timeout' => '30',
			]
		);

		$response_body = wp_remote_retrieve_body( $request );
		$response_code = wp_remote_retrieve_response_code( $request );

		if ( 200 !== $response_code ) {

			return new \WP_Error( 'error', esc_html__( 'We encountered an error. Please try again.', 'rw-elephant-rental-inventory' ) );

		}

		return rwe_jsonp_decode( $response_body );

	}

	/**
	 * Enqueue Scripts and Styles for single items.
	 *
	 * @since 2.0.0
	 *
	 * @return null
	 */
	public function enqueue_scripts() {

		$rtl    = ! is_rtl() ? '' : '-rtl';
		$suffix = SCRIPT_DEBUG ? '' : '.min';

		wp_enqueue_style( 'rwe-single-item', RW_ELEPHANT_RENTAL_INVENTORY_URL . "lib/assets/css/rw-elephant{$rtl}{$suffix}.css", [], RW_ELEPHANT_RENTAL_INVENTORY_VERSION, 'all' );

		wp_enqueue_script( 'stripe-checkout', 'https://checkout.stripe.com/checkout.js', [ 'jquery' ], '2.0.0', true );
		wp_enqueue_script( 'signature-pad', RW_ELEPHANT_RENTAL_INVENTORY_URL . "lib/assets/js/signature-pad{$suffix}.js", [], '3.0.0', true );
		wp_enqueue_script( 'rwe-elephant-invoice', RW_ELEPHANT_RENTAL_INVENTORY_URL . "lib/assets/js/rw-elephant-invoice{$suffix}.js", [ 'stripe-checkout', 'signature-pad' ], RW_ELEPHANT_RENTAL_INVENTORY_VERSION, true );

		wp_localize_script(
			'rwe-elephant-invoice',
			'rwElephant',
			[
				'ajaxurl'      => admin_url( 'admin-ajax.php' ),
				'_nonce_terms' => wp_create_nonce( 'rw_elephant_accept_terms' ),
				'_nonce_pay'   => wp_create_nonce( 'rw_elephant_submit_payment' ),
				'invoiceID'    => get_rwe_invoice_id(),
				'preloader'    => '<div class="rwe-circle-preloader">
					<div class="rwe-circle-preloader1 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader2 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader3 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader4 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader5 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader6 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader7 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader8 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader9 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader10 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader11 rwe-circle-preloader-child"></div>
					<div class="rwe-circle-preloader12 rwe-circle-preloader-child"></div>
				</div>',
				'stripe'       => [
					'publishableKey' => get_rwe_stripe_key(),
					'companyName'    => get_rwe_company_name(),
					'requireBilling' => (bool) rwe_invoice_billing_address_required(),
					'companyLogo'    => get_rwe_company_logo(),
					'btnText'        => esc_html__( 'Pay Invoice', 'rw-elephant-rental-inventory' ),
					'invalidAmount'  => __( 'Please select a valid amount.', 'rw-elephant-rental-inventory' ),
				],
			]
		);

		rwe_gallery_option_styles( 'rwe-single-item' );

	}

	/**
	 * Setup the invoice page.
	 *
	 * @since 2.0.0
	 */
	public function setup_invoice() {

		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ] );

		if ( ! is_rwe_invoice() ) {

			return;

		}

		// Add body class
		add_filter(
			'body_class',
			function( $classes ) {

				$classes[] = 'rwe-invoice';

				return $classes;

			}
		);

		add_filter( 'the_title', [ $this, 'invoice_title' ] );

		add_filter( 'the_content', [ $this, 'invoice_content' ], 10, 2 );

		// Disable comments
		add_filter( 'comments_open', '__return_false', 20, 2 );
		add_filter( 'ping_open', '__return_false', 20, 2 );
		add_filter( 'get_comments_number', '__return_zero', 10, 2 );
		add_filter( 'comments_template', '__return_null' );

		// Filter the page <title> tags
		add_filter(
			'wp_title',
			function( $title, $seperator, $seplocation ) {

				return __( 'Rental Order', 'rw-elephant-rental-inventory' );

			},
			10,
			3
		);

		// Disable the edit post link output
		add_filter( 'edit_post_link', '__return_false' );

	}

	/**
	 * Invoice page title.
	 *
	 * @since 2.0.0
	 *
	 * @return string Filtered invoce title
	 */
	public function invoice_title( $title ) {

		if ( is_admin() || ! in_the_loop() ) {

			return $title;

		}

		return esc_html__( 'Rental Order', 'rw-elephant-rental-inventory' );

	}

	/**
	 * Invoice page content.
	 *
	 * @since 2.0.0
	 *
	 * @param string  Original single content.
	 * @param integer Page ID.
	 *
	 * @return string Filtered invoice page content
	 */
	public function invoice_content( $content, $id = null ) {

		// Single item
		if ( is_admin() || ! get_rwe_invoice_id() || ! is_ssl() || is_rwe_invoice_error() ) {

			return $content;

		}

		if ( 0 === intval( self::$invoice_data['require_signature'] ) ) {

			?>

			<div class="rwe-inventory rwe-inventory--order-payment">

				<div class="rwe-inventory__main">

					<div class="rwe-order">

						<div class="rwe-order__detail">

							<div class="rwe-order-detail">

								<?php rwe_invoice_details(); ?>

							</div>

						</div>

						<?php Plugin::load_rw_template( 'invoice-pay' ); ?>

					</div>

				</div>

			</div>

			<?php

			return;

		}

		Plugin::load_rw_template( 'invoice-terms' );

	}

}
